// OwnerDrawStatic.cpp: Implementierungsdatei
//

#include "stdafx.h"
#include "BitmapStatic.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COwnerDrawStatic

CBitmapStatic::CBitmapStatic()
{
  m_bTransparent = false;
}

CBitmapStatic::~CBitmapStatic()
{
}

bool CBitmapStatic::LoadBitmap(LPCTSTR szPathName)
{
  HBITMAP hNewBitmap = (HBITMAP)::LoadImage(NULL, szPathName,IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_CREATEDIBSECTION | LR_DEFAULTSIZE);
  if (hNewBitmap)
    SetBitmap(hNewBitmap);
  return hNewBitmap?true:false;
}

void CBitmapStatic::SetTransparent(bool bSet /*=true*/)
{
  m_bTransparent = bSet;
}

void CBitmapStatic::DrawTransparent(CDC * pDC,CRect rcDest)
{
  BITMAP     bm;
  COLORREF   cColor;
  CBitmap    bmAndBack, bmAndObject, bmAndMem, bmSave;
  CBitmap    *bmBackOld, *bmObjectOld, *bmMemOld, *bmSaveOld;
  CDC        dcMem, dcBack, dcObject, dcTemp;
  POINT      ptSize;

  // Prfen, ob eine Bitmap zugewiesen ist
  if (!GetBitmap())
    return;
  CBitmap *  pOrigBitmap = CBitmap::FromHandle(GetBitmap());

  COLORREF cTransparentColor = RGB(255,255,255);

  pOrigBitmap->GetBitmap(&bm);

  ptSize.x = bm.bmWidth;            // Get width of bitmap
  ptSize.y = bm.bmHeight;           // Get height of bitmap
  pDC->DPtoLP(&ptSize, 1);      // Convert from device
                                // to logical points

  DWORD Size = bm.bmWidthBytes * bm.bmHeight;

  char * NewBits;
  NewBits = new char[Size+24];
  pOrigBitmap->GetBitmapBits(Size,NewBits);
  
  delete[]NewBits;

  bmSave.CreateCompatibleBitmap(pDC, ptSize.x, ptSize.y);

  dcTemp.CreateCompatibleDC(pDC);
  bmSaveOld = dcTemp.SelectObject(&bmSave);   // Select the bitmap

  // Bitmap in Screen Kompatible Bitmap kopieren
  {
	  CDC mdc; // memory DC
	  mdc.CreateCompatibleDC(&dcTemp);
	  CBitmap* pOld = mdc.SelectObject(pOrigBitmap);
	  dcTemp.BitBlt( 0, 0, bm.bmWidth, bm.bmHeight, &mdc, 0, 0, SRCCOPY);
	  mdc.SelectObject(pOld);
  }

  bmSave.GetBitmap(&bm);
  Size = bm.bmWidthBytes * bm.bmHeight;

  NewBits = new char[Size+24];
  bmSave.GetBitmapBits(Size,NewBits);

  delete[]NewBits;

  // Create some DCs to hold temporary data.
  dcBack   .CreateCompatibleDC(pDC);
  dcObject .CreateCompatibleDC(pDC);
  dcMem    .CreateCompatibleDC(pDC);

  // Create a bitmap for each DC. DCs are required for a number of
  // GDI functions.

  // Monochrome DC
  bmAndBack.CreateBitmap(ptSize.x, ptSize.y, 1, 1, NULL);

  // Monochrome DC
  bmAndObject.CreateBitmap(ptSize.x, ptSize.y, 1, 1, NULL);

  bmAndMem   .CreateCompatibleBitmap(pDC, ptSize.x, ptSize.y);

  // Each DC must select a bitmap object to store pixel data.
  bmBackOld   = dcBack  .SelectObject(&bmAndBack);
  bmObjectOld = dcObject.SelectObject(&bmAndObject);
  bmMemOld    = dcMem   .SelectObject(&bmAndMem);

  // Set proper mapping mode.
  dcTemp.SetMapMode(pDC->GetMapMode());

  // Set the background color of the source DC to the color.
  // contained in the parts of the bitmap that should be transparent
  cColor = dcTemp.SetBkColor(cTransparentColor);

  // Create the object mask for the bitmap by performing a BitBlt
  // from the source bitmap to a monochrome bitmap.
  dcObject.BitBlt(0, 0, ptSize.x, ptSize.y, &dcTemp, 0, 0,SRCCOPY);

  // Set the background color of the source DC back to the original
  // color.
  dcTemp.SetBkColor( cColor);

  // Create the inverse of the object mask.
  dcBack.BitBlt(0, 0, ptSize.x, ptSize.y, &dcObject, 0, 0, NOTSRCCOPY);

  // Copy the background of the main DC to the destination.
  dcMem.BitBlt(0, 0, ptSize.x, ptSize.y, pDC, rcDest.left,rcDest.top,SRCCOPY);

  // Mask out the places where the bitmap will be placed.
  dcMem.BitBlt(0, 0, ptSize.x, ptSize.y, &dcObject, 0, 0, SRCAND);

  // Mask out the transparent colored pixels on the bitmap.
  dcTemp.BitBlt(0, 0, ptSize.x, ptSize.y, &dcBack, 0, 0, SRCAND);

  // XOR the bitmap with the background on the destination DC.
  dcMem.BitBlt( 0, 0, ptSize.x, ptSize.y, &dcTemp, 0, 0, SRCPAINT);

  // Copy the destination to the screen.
  pDC->BitBlt(rcDest.left,rcDest.top, ptSize.x, ptSize.y, &dcMem, 0, 0,SRCCOPY);

  // Delete the memory bitmaps.
  dcBack  .SelectObject(bmBackOld);
  dcObject.SelectObject(bmObjectOld);
  dcMem   .SelectObject(bmMemOld);
  dcTemp  .SelectObject(bmSaveOld);

  bmAndBack.DeleteObject();
  bmAndObject.DeleteObject();
  bmAndMem.DeleteObject();
  bmSave.DeleteObject();
  return;
} 

BEGIN_MESSAGE_MAP(CBitmapStatic, CStatic)
	//{{AFX_MSG_MAP(CBitmapStatic)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Behandlungsroutinen fr Nachrichten CBitmapStatic 

void CBitmapStatic::OnPaint() 
{
  if (m_bTransparent || (GetExStyle()&WS_EX_TRANSPARENT))
  {
	  CPaintDC dc(this); // device context for painting
    CRect rcClient;
    GetClientRect(rcClient);
    DrawTransparent(&dc,rcClient);
  }
  else
    CStatic::OnPaint();
}
