// BlockDlg.cpp : Implementation file
//

#include "stdafx.h"
#include "Block.h"
#include "BlockDlg.h"
#include "TS_LF_Import.h"

#include "label.h"
#include "BitmapStatic.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#pragma comment(lib, "version.lib")

// Set TRANSPONDER_TYPE to the type of Transponder which you want to use.
// Valid Transponder types are given in TS_RW_Import.h
// TTYP_HITAG_2   TTYP_HITAG_S   
// TTYP_Q5        TTYP_EM4569  
// TTYP_EM4305    TTYP_ATA5577

#define TRANSPONDER_TYPE TTYP_HITAG_S

// The best filter value depends on the transponder type used.
#if   (TRANSPONDER_TYPE == TTYP_HITAG_2)
#define BEST_FILTER 0x4A
#elif (TRANSPONDER_TYPE == TTYP_HITAG_S)
#define BEST_FILTER 0x4A
#elif (TRANSPONDER_TYPE == TTYP_Q5)
#define BEST_FILTER 0x4B
#elif (TRANSPONDER_TYPE == TTYP_EM4569)
#define BEST_FILTER 0x40
#elif (TRANSPONDER_TYPE == TTYP_EM4305)
#define BEST_FILTER 0x40
#elif (TRANSPONDER_TYPE == TTYP_ATA5577)
#define BEST_FILTER 0x4B
#endif

CString GetProgVersion ()
{
  char * abData;
  char String[255];
  CString Version;
  DWORD  handle = NULL;
  UINT   wSize = 0;
  DWORD  dwMax = 0;
  LPSTR  lpBuffer = NULL;
  char ExeName[_MAX_PATH];
  ::GetModuleFileName(NULL,ExeName,_MAX_PATH);
  dwMax = GetFileVersionInfoSize(ExeName, &handle);
  if (dwMax == 0)
  {
    Version = "1.xx";
    return Version;
  }
  if (dwMax > 32000)
  {
    Version = "1.xx";
    return Version;
  }
  
  abData = new char[dwMax+10];
  memset (abData,0,(WORD)(dwMax+10));
  if (GetFileVersionInfo(ExeName, handle, dwMax, abData))
  {
    strcpy_s (String,"\\StringFileInfo\\040704b0\\ProductVersion");
    if (VerQueryValue(abData, String, (LPVOID *)&lpBuffer, &wSize))
    {
      if ((lpBuffer) && (wSize!=0) && (strlen(lpBuffer) < 20))
        Version = lpBuffer;
    }
  }
  if (Version.IsEmpty())
    Version = "1.xx";
  delete []abData;
  return Version;
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg-Dialog box for "Info"

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CBitmapStatic	m_GisBitmap;
  CLabel m_Email;
  CLabel m_URL;
	//}}AFX_DATA

	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV-Untersttzung
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Control(pDX, IDC_GIS_LOGO, m_GisBitmap);
	DDX_Control(pDX, IDC_EMAIL, m_Email);
	DDX_Control(pDX, IDC_URL, m_URL);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CAboutDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

  // URL und Email als Link darstellen (Blau, unterstrichen)
  LOGFONT lf;
  GetFont()->GetLogFont(&lf);
  lf.lfUnderline = TRUE;
  m_Email.SetFont(&lf);
  m_Email.SetLink(TRUE);
  m_Email.SetTextColor(RGB(  0,  0,255));
  m_Email.SetLinkCursor(::LoadCursor(NULL,MAKEINTRESOURCE(32649)));
  m_URL  .SetFont(&lf);
  m_URL  .SetLink(TRUE);
  m_URL  .SetTextColor(RGB(  0,  0,255));
  m_URL  .SetLinkCursor(::LoadCursor(NULL,MAKEINTRESOURCE(32649)));

  m_GisBitmap.SetTransparent();

  // show correct version information
  CString text;
  GetDlgItemText(IDC_VERSION,text);
  
  text += " ";
  text += GetProgVersion();
  SetDlgItemText (IDC_VERSION,text);
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX-Eigenschaftenseiten sollten FALSE zurckgeben
}

/////////////////////////////////////////////////////////////////////////////
// CBlockDlg Dialogfeld

CBlockDlg::CBlockDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBlockDlg::IDD, pParent)
  , m_nBlockNr(0)
{
	//{{AFX_DATA_INIT(CBlockDlg)
	m_BlockData = _T("");
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
  m_nPortHandle = NULL;
  m_nError = 0;
}

void CBlockDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);

  //{{AFX_DATA_MAP(CBlockDlg)
  DDX_Text(pDX, IDC_BLOCK_DATA, m_BlockData);
  DDV_MaxChars(pDX, m_BlockData, 8);
  DDX_Text(pDX, IDC_BLOCK_NR, m_nBlockNr);
	DDV_MinMaxInt(pDX, m_nBlockNr, 0, 255);
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CBlockDlg, CDialog)
	//{{AFX_MSG_MAP(CBlockDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_READ_BLOCK, OnReadBlock)
	ON_BN_CLICKED(IDC_WRITE_BLOCK, OnWriteBlock)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBlockDlg Message-Handler

BOOL CBlockDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{	
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	SetIcon(m_hIcon, TRUE);			// for big Symbol
	SetIcon(m_hIcon, FALSE);		// for small Symbol
	
  // Get List of all USB Devices
  int nMaxSize = 255;
	char USBNamesBuffer[255];
  CStringList USBListe;
  CString strName;
	int nLen = TSLF_GetUSBDeviceNames(USBNamesBuffer,nMaxSize);
  int nStrLen = 0;
  do
  {
    strName = USBNamesBuffer + nStrLen;
    nStrLen += strName.GetLength() + 1;
    if(strName.GetLength() != 0)
      USBListe.AddTail(strName);
  }while(strName.GetLength() != 0);

	// Use first USB device and open port
  // the port has to be closed when application closes
  // This is done in OnDestroy() .
  // If no USB-Device is found, the application tries to open a COM Port.
  m_nPortHandle = -1;
  if(USBListe.GetCount() > 0)
  {
    m_nPortHandle = TSLF_Open(USBListe.GetHead(),19200,0,1000);
    if (m_nPortHandle > 0)
    {
      // Get device version from device, to check if read device is connected.
      // The first 3 digits of the string give the device number
      // valid device number is 039 for TS-W34AC
      char Version[4],DeviceName[32];
      int nReturn = TSLF_GetDeviceVersion(m_nPortHandle, Version, 4,DeviceName,32);
      if(nReturn == -1)
      {
        int Error = TSLF_GetLastError(m_nPortHandle);
        TSLF_Close(m_nPortHandle);
        m_nPortHandle = -1;
      }
      else
      {
        if (TSLF_IsProgrammer(m_nPortHandle) != 1)
        {
          // Invalid device, will be closed
          AfxMessageBox("Connected device is no programmer! Please use a programmer instead");
          int Error = TSLF_GetLastError(m_nPortHandle);
          TSLF_Close(m_nPortHandle);
          m_nPortHandle = -1;
        }
      }
    }
  }
  else
  {
    // Try connection using a COM port
    CString com = "COM1";
    int nMaxSize = 255;
    // First get list of com ports
	  char * COMNamesBuffer = new char[nMaxSize];
    CStringList COMListe;
    CString strName;
	  int nLen = TSLF_GetCOMDeviceNames(COMNamesBuffer,nMaxSize);
    int nStrLen = 0;
    do
    {
      strName = COMNamesBuffer + nStrLen;
      nStrLen += strName.GetLength() + 1;
      if(strName.GetLength() != 0)
        COMListe.AddTail(strName);
    }while(strName.GetLength() != 0);
    delete COMNamesBuffer;

    // then try all found ports
    POSITION Pos = COMListe.GetHeadPosition();
    while (Pos)
    {
      m_nPortHandle = TSLF_Open(COMListe.GetNext(Pos),19200,0,1000);
      if (m_nPortHandle > 0)
      {
        // Get device version from device, to cehck if read device is connected.
        // The first 3 digits of the string give the device number
        // valid device number is 039 for TS-W34AC
        char Version[4],DeviceName[32];
        int nReturn = TSLF_GetDeviceVersion(m_nPortHandle, Version, 4,DeviceName,32);
        if(nReturn == -1)
        {
          int Error = TSLF_GetLastError(m_nPortHandle);
          TSLF_Close(m_nPortHandle);
          m_nPortHandle = -1;
        }
        else
        {
          if (TSLF_IsProgrammer(m_nPortHandle) != 1)
          {
            // Invalid device, will be closed
            AfxMessageBox("Connected device is no programmer! Please use a programmer instead");
            int Error = TSLF_GetLastError(m_nPortHandle);
            TSLF_Close(m_nPortHandle);
            m_nPortHandle = -1;
          }
          break;
        }
      }
    }
  }

  if (m_nPortHandle <= 0)
    AfxMessageBox("No connection to device");
  else
  {
    // The best filter value depends on the transponder type used.
    // The filter has to be set one time. It is stored in the EEProm of the reader,
    // So the filter does not to be changed if only one transponder type is used.
    // At new devices (TS_RW38 and later) this is not needed, the filters for all transponder types are already stored in the device.
    // At old devices the given transponder type is not used, because there is only one filter value stored.
    BYTE ucFilter = BEST_FILTER;
    TSLF_SetFilter(m_nPortHandle,TRANSPONDER_TYPE,&ucFilter,1);
  }
	return TRUE;  
}

void CBlockDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

void CBlockDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this);

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

HCURSOR CBlockDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CBlockDlg::OnDestroy()
{
  // Close Port if open
  if(m_nPortHandle)
  {
    TSLF_Close(m_nPortHandle);
    m_nPortHandle = 0;
  }

	CDialog::OnDestroy();
}

BYTE MakeHex(CString str)
{
  return (BYTE)strtol(str,NULL,16);
}

bool CBlockDlg::Select()
{
#if   (TRANSPONDER_TYPE == TTYP_HITAG_1)
  BYTE UID_Data[4];
  if (TSLF_UID_Request(m_nPortHandle, TTYP_HITAG_1, UID_Data, 4) == -1)
  {
    CString ErrMsg;
    ErrMsg.Format("Error %d at CCNew", TSLF_GetLastError(m_nPortHandle));
    AfxMessageBox(ErrMsg);
    return false;
  }
  BYTE Config[4];
  if (TSLF_Select(m_nPortHandle, TTYPE_HITAG_1, UID_Data, 4, Config, 4) == -1)
  {
    CString ErrMsg;
    ErrMsg.Format("Error %d at UID Select", TSLF_GetLastError(m_nPortHandle));
    AfxMessageBox(ErrMsg);
    return false;
  }
#elif (TRANSPONDER_TYPE == TTYP_HITAG_2)
  BYTE UID_Data[4];
  if (TSLF_UID_Request(m_nPortHandle, TTYP_HITAG_2, UID_Data, 4) == -1)
  {
    CString ErrMsg;
    ErrMsg.Format("Error %d at StartAuth", TSLF_GetLastError(m_nPortHandle));
    AfxMessageBox(ErrMsg);
    return false;
  }
  BYTE Password[4] = { 0x52,0x4B,0x49,0x4D }; // Use default password
  BYTE TagPW[4];
  if (TSLF_Select(m_nPortHandle, TTYP_HITAG_2, Password, 4, TagPW, 4) == -1)
  {
    CString ErrMsg;
    ErrMsg.Format("Error %d at Select", TSLF_GetLastError(m_nPortHandle));
    AfxMessageBox(ErrMsg);
    return false;
  }
#elif (TRANSPONDER_TYPE == TTYP_HITAG_S)
  BYTE UID_Data[4];
  if (TSLF_UID_Request(m_nPortHandle, TTYP_HITAG_S, UID_Data, 4) == -1)
  {
    CString ErrMsg;
    ErrMsg.Format("Error %d at UID Request", TSLF_GetLastError(m_nPortHandle));
    AfxMessageBox(ErrMsg);
    return false;
  }
  BYTE Config[4];
  if (TSLF_Select(m_nPortHandle, TTYP_HITAG_S, UID_Data, 4, Config, 4) == -1)
  {
    CString ErrMsg;
    ErrMsg.Format("Error %d at UID Select", TSLF_GetLastError(m_nPortHandle));
    AfxMessageBox(ErrMsg);
    return false;
  }
#endif

  // All other Transponder types do not need to be selected,
  // so we just return true to signal all is ok.
  return true;
}

void CBlockDlg::OnReadBlock()
{
  if (m_nPortHandle <= 0)
  {
    AfxMessageBox("No Device open");
    return;
  }

  UpdateData(TRUE);

  if (!Select())
    return;

  DWORD BlockData = 0;
  if (TSLF_Read(m_nPortHandle, TRANSPONDER_TYPE, m_nBlockNr, (BYTE*)&BlockData,4) != -1)
  {
    // Read successfully done
    m_BlockData.Format("%08X",BlockData);
  }
  else
  {
    CString strMsg;
    strMsg.Format("Error %d reading Block data",TSLF_GetLastError(m_nPortHandle));
    AfxMessageBox(strMsg);
  }

  UpdateData(FALSE);
}

void CBlockDlg::OnWriteBlock()
{
  if (m_nPortHandle <= 0)
  {
    AfxMessageBox("No Device open");
    return;
  }

  UpdateData(TRUE);

  if (!Select())
    return;

  // Get Block Code from Editbox
  // It is used as hexadecimal coding here

  DWORD BlockData = strtoul(m_BlockData,NULL,16);


  // We have to give 5 as size because only lowest 5 Byte contain Block Data.
  if (TSLF_Write(m_nPortHandle, TRANSPONDER_TYPE, m_nBlockNr, (BYTE*)&BlockData, 4, 0) != -1)
  {
    CString strMsg;
    strMsg.Format("Block data written to Transponder");
    AfxMessageBox(strMsg);
  }
  else
  {
    CString strMsg;
    strMsg.Format("Error %d writing Block Data",TSLF_GetLastError(m_nPortHandle));
    AfxMessageBox(strMsg);
  }

  UpdateData(FALSE);
}
