﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using rfid;

namespace AnimalCS
{
  public partial class Form1 : Form
  {
    int TTYP = GIS_LF_API.TTYP_EL9265;

    static string MakeStringFromArray(sbyte[] Arr,int StringNummer)
    {
      string strLine;
      strLine = "";

      int i = 0;
      for (int aktStringNummer = 0; aktStringNummer <= StringNummer; aktStringNummer++)
      {
        strLine = "";
        for (; (i < Arr.Length); i++)
        {
          if (Arr[i] == 0)
          {
            break;
          }
          strLine += (char)Arr[i];
        }
        i++;
        if (strLine.Length == 0)
          break;
      }
      return strLine;
    }

    static sbyte[] MakeArrayFromString(string Name)
    {
      sbyte[] arr = new sbyte[Name.Length];
      for (int i = 0; i < Name.Length; i++)
        arr[i] = (sbyte)Name[i];
      return arr;
    }

    static void CopyArrayTo(SByte[] Src, int SrcOffset, SByte[] Dest, int DestOffset, int Len)
    {
      while (Len > 0)
      {
        Dest[DestOffset++] = Src[SrcOffset++];
        Len--;
      }
    }

    static Int64 BufferToInt64(byte[] Arr)
    {
      Int64 Result = 0;
      for (int i = Arr.Length - 1; i >= 0; i--)
      {
        Result <<= 8;
        Result += Arr[i];
      }
      return Result;
    }

    static byte[] Int64ToBuffer(Int64 Value)
    {
      byte[] Buffer = new byte[8];
      for (int i = 0; i < 8; i++)
      {
        Buffer[i] = (byte)(Value & 0x00FF);
        Value >>= 8;
      }
      return Buffer;
    }

    static bool IsNumber(string value)
    {
      if (value.Length == 0)
        return false;

      for (int i = 0; i < value.Length; i++)
        if (!Char.IsDigit(value[i]))
          return false;

      return true;
    }

    int PortHandle;

    public Form1()
    {
      InitializeComponent();
      PageNr.Text = "8";

      ConnectDevice();
    }

    private bool ConnectDevice()
    {
      int MaxLen = 255;
      int ListLen = 0;
      int DevListLen;

      SByte[] NamenListe = new SByte[1000];
      SByte[] DevList = new SByte[MaxLen];

      if (PortHandle > 0)
      {
        GIS_LF_API.TSLF_Close(PortHandle);
        PortHandle = 0;
      }

      DevListLen = GIS_LF_API.TSLF_GetUSBDeviceNames(DevList, MaxLen) - 1;
      CopyArrayTo(DevList, 0, NamenListe, ListLen, DevListLen);
      ListLen += DevListLen;
      DevListLen = GIS_LF_API.TSLF_GetLanDeviceNames(DevList, MaxLen) - 1;
      CopyArrayTo(DevList, 0, NamenListe, ListLen, DevListLen);
      ListLen += DevListLen;
      DevListLen = GIS_LF_API.TSLF_GetCOMDeviceNames(DevList, MaxLen) - 1;
      CopyArrayTo(DevList, 0, NamenListe, ListLen, DevListLen);
      ListLen += DevListLen;
      ListLen++;

      if (ListLen > 1)
      {
        string DeviceName;
        // Use first Device in List
        DeviceName = MakeStringFromArray(NamenListe, 0);

        PortHandle = GIS_LF_API.TSLF_Open(MakeArrayFromString(DeviceName), 19200, 0, 500);
        if (GIS_LF_API.TSLF_IsProgrammer(PortHandle) != 1)
        {
          Message.Text = "Device is no Programmer";
          GIS_LF_API.TSLF_Close(PortHandle);
          PortHandle = 0;
        }
        else
        {
          // Get device version from device, to cehck if read device is connected.
          // The first 3 digits of the string give the device number
          // valid device number is 039 for TS-W34AC, 244 for TS-RW38AC
          sbyte[] Version = new sbyte[4];
          sbyte[] InternalDeviceName = new sbyte[32];
          int nReturn = GIS_LF_API.TSLF_GetDeviceVersion(PortHandle, Version, 4,InternalDeviceName,32);
          if(nReturn == -1)
          {
            int Error = GIS_LF_API.TSLF_GetLastError(PortHandle);
            Message.Text = "DeviceVersion can not be read";
            GIS_LF_API.TSLF_Close(PortHandle);
            PortHandle = 0;
          }
          else
          {
            if (GIS_LF_API.TSLF_IsAnimalProgrammer(PortHandle) != 1)
            {
              // Invalid device, will be closed
              Message.Text = "Connected device is no Animal programmer!";
              GIS_LF_API.TSLF_Close(PortHandle);
              PortHandle = 0;
            }
          }
        }
      }
      else
      {
        PortHandle = 0;
      }

      if (PortHandle <= 0)
      {
        Message.Text = "Device could not be opened";
      }
      return PortHandle > 0;
    }

    private void Close_Click(object sender, EventArgs e)
    {
      if (PortHandle > 0)
      {
        GIS_LF_API.TSLF_Close(PortHandle);
        PortHandle = 0;
      }
      Close();
    }

    private void ReadBtn_Click(object sender, EventArgs e)
    {
      Message.Text = "";
      CountryCode.Text = "";
      IdentNr.Text = "";
      AnimalTemp.Text = "";

      if (PortHandle <= 0)
      {
        if (!ConnectDevice())
        {
          Message.Text = "No Device attached";
          return;
        }
      }

      // During Animal read the transponder must be not selected!
      // So we do a Reset of the transponder before
      if (GIS_LF_API.TSLF_ResetTransponder(PortHandle, TTYP) == -1)
      {
        Message.Text = "ResetTransponder Error " + GIS_LF_API.TSLF_GetLastError(PortHandle).ToString();
      }

      int Country;
      Byte[] Buffer = new Byte[8];
      int AnimalFlag;
      int DatenblockFlag;
      int Reserved;
      int Extension;
      if (GIS_LF_API.TSLF_Read_FDXB(PortHandle, out Country, Buffer, 8, out AnimalFlag, out DatenblockFlag, out Reserved, out Extension) >= 0)
      {
        CountryCode.Text = Country.ToString();
        IdentNr.Text = BufferToInt64(Buffer).ToString();
        if (Extension != 0)
        {
          double fTemp = GIS_LF_API.TSLF_ConvertTemp(PortHandle, TTYP, Extension);
          AnimalTemp.Text = (fTemp/65536).ToString("F3");
        }
      }
      else
      {
        Message.Text = "Read Error " + GIS_LF_API.TSLF_GetLastError(PortHandle).ToString();
      }
    }

    private void WriteBtn_Click(object sender, EventArgs e)
    {
      Message.Text = "";
      if (PortHandle <= 0)
      {
        if (!ConnectDevice())
        {
          Message.Text = "No Device attached";
          return;
        }
      }

      Byte[] FactoryID = new Byte[8];
      if (IsNumber(IdentNr.Text))
      {
        Byte[] Buffer = Int64ToBuffer(Convert.ToInt64(IdentNr.Text));

        int Country = 0;
        if (IsNumber(CountryCode.Text))
          Country = Convert.ToInt32(CountryCode.Text);
        else
          CountryCode.Text = "";

        FactoryID[0] = 0;
        FactoryID[1] = 0;
        FactoryID[2] = 0;
        FactoryID[3] = 0;
        FactoryID[4] = 0;
        FactoryID[5] = 0;
        FactoryID[6] = 0;
        FactoryID[7] = 0;

        if (GIS_LF_API.TSLF_Write_FDXB(PortHandle, TTYP, FactoryID, Country, Buffer, 8, 1, 0, 0, 0, 0) < 0)
        {
          Message.Text = "Write Error " + GIS_LF_API.TSLF_GetLastError(PortHandle).ToString();
        }
        else
        {
          Message.Text = "Write OK";
        }

        // Reset the Antenna Field so Transponder is deselected to make next Read work again
        GIS_LF_API.TSLF_ResetTransponder(PortHandle, 0);
      }
      else
      {
        Message.Text = "Enter Ident Nr. (only Digits allowed)";
      }
    }

    private void MeasureBtn_Click(object sender, EventArgs e)
    {
      int Temp;
      if (GIS_LF_API.TSLF_GetTempGrad(PortHandle, TTYP,out Temp) != GIS_LF_API.TSLF_NO_ERROR)
      {
        Message.Text = "Error " + GIS_LF_API.TSLF_GetLastError(PortHandle).ToString() + " reading Temp";

        if (GIS_LF_API.TSLF_GetLastError(PortHandle) == GIS_LF_API.TSLF_PORTHANDLE_UNGUELTIG)
          PortHandle = 0;
      }
      else
      {
        double fTemp = Temp;
        MeasureTemp.Text = (fTemp/65536).ToString("F3");
      }
    }

    private void StoreBtn_Click(object sender, EventArgs e)
    {
      Message.Text = "";
      if (PortHandle <= 0)
      {
        if (!ConnectDevice())
        {
          Message.Text = "No Device open";
          return;
        }
      }

      MeasureTemp.Text = "";

      int BlockNr;
      if (IsNumber(PageNr.Text))
      {
        BlockNr = Convert.ToInt32(PageNr.Text);
        if ((BlockNr < 8) || (BlockNr >57))
        {
          Message.Text = "Block out of Range! 8 - 57 allowed";
          return;
        }
      }
      else
      {
        BlockNr = 8;
        PageNr.Text = "8";
      }

      int SensorData;
      if (GIS_LF_API.TSLF_StoreTempSensor(PortHandle, TTYP, BlockNr, BlockNr, out SensorData) == GIS_LF_API.TSLF_RETVAL_OK)
      {
        double fTemp = GIS_LF_API.TSLF_ConvertTemp(PortHandle, TTYP, SensorData);
        MeasureTemp.Text = (fTemp/65536).ToString("F3");
      }
      else
      {
        Message.Text = "Error " + GIS_LF_API.TSLF_GetLastError(PortHandle).ToString() + " reading Temp";
        if (GIS_LF_API.TSLF_GetLastError(PortHandle) == GIS_LF_API.TSLF_PORTHANDLE_UNGUELTIG)
          PortHandle = 0;
      }
    }

    private void DeactivateBtn_Click(object sender, EventArgs e)
    {
      Message.Text = "";
      if (PortHandle <= 0)
      {
        if (!ConnectDevice())
        {
          Message.Text = "No Device open";
          return;
        }
      }

      if (GIS_LF_API.TSLF_ActivateTempSensor(PortHandle,TTYP,0) == GIS_LF_API.TSLF_RETVAL_ERROR)
      {
        Message.Text = "deactivating Temp Sensor Error " + GIS_LF_API.TSLF_GetLastError(PortHandle).ToString();
        if (GIS_LF_API.TSLF_GetLastError(PortHandle) == GIS_LF_API.TSLF_PORTHANDLE_UNGUELTIG)
          PortHandle = 0;
        return;
      }
      Message.Text = "Temp Sensor Deactivated";

    }

    private void ActivateBtn_Click(object sender, EventArgs e)
    {
      Message.Text = "";
      if (PortHandle <= 0)
      {
        if (!ConnectDevice())
        {
          Message.Text = "No Device open";
          return;
        }
      }

      int BlockNr;
      if (IsNumber(PageNr.Text))
      {
        BlockNr = Convert.ToInt32(PageNr.Text);
        if ((BlockNr < 8) || (BlockNr >57))
        {
          Message.Text = "Block out of Range! 8 - 57 allowed";
          return;
        }
      }
      else
      {
        BlockNr = 8;
        PageNr.Text = "8";
      }

      if (GIS_LF_API.TSLF_ActivateTempSensor(PortHandle,TTYP,BlockNr) == GIS_LF_API.TSLF_RETVAL_ERROR)
      {
        Message.Text = "activating Temp Sensor Error " + GIS_LF_API.TSLF_GetLastError(PortHandle).ToString();
        if (GIS_LF_API.TSLF_GetLastError(PortHandle) == GIS_LF_API.TSLF_PORTHANDLE_UNGUELTIG)
          PortHandle = 0;
        return;
      }
      Message.Text = "Temp Sensor Activated";

    }
  }
}
