﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using rfid;

namespace FskCS
{
  public partial class Form1 : Form
  {
    int TTYP = GIS_LF_API.TTYP_ATA5577;

    static string MakeStringFromArray(sbyte[] Arr,int StringNummer)
    {
      string strLine;
      strLine = "";

      int i = 0;
      for (int aktStringNummer = 0; aktStringNummer <= StringNummer; aktStringNummer++)
      {
        strLine = "";
        for (; (i < Arr.Length); i++)
        {
          if (Arr[i] == 0)
          {
            break;
          }
          strLine += (char)Arr[i];
        }
        i++;
        if (strLine.Length == 0)
          break;
      }
      return strLine;
    }

    static sbyte[] MakeArrayFromString(string Name)
    {
      sbyte[] arr = new sbyte[Name.Length];
      for (int i = 0; i < Name.Length; i++)
        arr[i] = (sbyte)Name[i];
      return arr;
    }

    static void CopyArrayTo(SByte[] Src, int SrcOffset, SByte[] Dest, int DestOffset, int Len)
    {
      while (Len > 0)
      {
        Dest[DestOffset++] = Src[SrcOffset++];
        Len--;
      }
    }

    static bool IsNumber(string value)
    {
      if (value.Length == 0)
        return false;

      for (int i = 0; i < value.Length; i++)
        if (!Char.IsDigit(value[i]))
          return false;

      return true;
    }

    int PortHandle;

    public Form1()
    {
      InitializeComponent();

      int MaxLen = 255;
      int ListLen = 0;
      int DevListLen;

      SByte[] NamenListe = new SByte[1000];
      SByte[] DevList = new SByte[MaxLen];

      DevListLen = GIS_LF_API.TSLF_GetUSBDeviceNames(DevList, MaxLen) - 1;
      CopyArrayTo(DevList, 0, NamenListe, ListLen, DevListLen);
      ListLen += DevListLen;
      DevListLen = GIS_LF_API.TSLF_GetLanDeviceNames(DevList, MaxLen) - 1;
      CopyArrayTo(DevList, 0, NamenListe, ListLen, DevListLen);
      ListLen += DevListLen;
      DevListLen = GIS_LF_API.TSLF_GetCOMDeviceNames(DevList, MaxLen) - 1;
      CopyArrayTo(DevList, 0, NamenListe, ListLen, DevListLen);
      ListLen += DevListLen;
      ListLen++;

      if (ListLen > 1)
      {
        string DeviceName;
        // Use first Device in List
        DeviceName = MakeStringFromArray(NamenListe, 0);

        PortHandle = GIS_LF_API.TSLF_Open(MakeArrayFromString(DeviceName), 19200, 0, 500);
        if (GIS_LF_API.TSLF_IsProgrammer(PortHandle) != 1)
        {
          MessageBox.Show("Device is no Programmer");
          GIS_LF_API.TSLF_Close(PortHandle);
          PortHandle = 0;
        }
        else
        {
          // Get device version from device, to cehck if read device is connected.
          // The first 3 digits of the string give the device number
          // valid device number is 244 for TS-RW38AC
          sbyte[] Version = new sbyte[4];
          sbyte[] InternalDeviceName = new sbyte[32];
          int nReturn = GIS_LF_API.TSLF_GetDeviceVersion(PortHandle, Version, 4,InternalDeviceName,32);
          if(nReturn == -1)
          {
            int Error = GIS_LF_API.TSLF_GetLastError(PortHandle);
            MessageBox.Show("DeviceVersion can not be read");
            GIS_LF_API.TSLF_Close(PortHandle);
            PortHandle = 0;
          }
          else
          {
            if (GIS_LF_API.TSLF_IsPlusProgrammer(PortHandle) != 1)
            {
              // Invalid device, will be closed
              MessageBox.Show("Connected device is no Fsk programmer!");
              GIS_LF_API.TSLF_Close(PortHandle);
              PortHandle = 0;
            }
          }
        }
      }
      else
      {
        PortHandle = 0;
      }

      if (PortHandle <= 0)
      {
        MessageBox.Show("Device could not be opened");
      }
    }

    private void Close_Click(object sender, EventArgs e)
    {
      if (PortHandle > 0)
        GIS_LF_API.TSLF_Close(PortHandle);
      Close();
    }

    private void Read_Click(object sender, EventArgs e)
    {
      if (PortHandle > 0)
      {
        uint Manufacturer = 0;
        uint iIdentNr = 0;
        uint Fixed = 0;
        byte[] HugeArr = new Byte[8];

        int RetVal;

        if (Radio26Bit.Checked)
          RetVal = GIS_LF_API.TSLF_Read_FSK26Bit(PortHandle, out Manufacturer, out iIdentNr);
        else if (Radio34Bit.Checked)
          RetVal = GIS_LF_API.TSLF_Read_FSK34Bit(PortHandle, out Manufacturer, out iIdentNr);
        else if (Radio36Bit.Checked)
          RetVal = GIS_LF_API.TSLF_Read_FSK36Bit(PortHandle, out Manufacturer, out iIdentNr, out Fixed);
        else if (Radio37Bit.Checked)
          RetVal = GIS_LF_API.TSLF_Read_FSK37BitHuge(PortHandle, HugeArr,HugeArr.Length);
        else
        {
          MessageBox.Show("No Bit size selected!");
          return;
        }

        if (RetVal >= 0)
        {
          if (Radio37Bit.Checked)
          {
            Int64 HugeVal = BitConverter.ToInt64(HugeArr, 0);

            IdentNr.Text = HugeVal.ToString();
            ManufacturerCode.Text = "";
          }
          else
          {
            ManufacturerCode.Text = Manufacturer.ToString();
            IdentNr.Text = iIdentNr.ToString();
          }
        }
        else
        {
          MessageBox.Show("Read Error");
        }
      }
      else
      {
        MessageBox.Show("No Device attached");
      }
    }

    private void Write_Click(object sender, EventArgs e)
    {
      if (PortHandle > 0)
      {
        uint iIdentNr = IsNumber(IdentNr.Text)?Convert.ToUInt32(IdentNr.Text):0;
        uint Manufacturer = IsNumber(ManufacturerCode.Text) ? Convert.ToUInt32(ManufacturerCode.Text):0 ;
        Int64 HugeVal = IsNumber(IdentNr.Text)?Convert.ToInt64(IdentNr.Text):0;

        byte[] HugeArr = BitConverter.GetBytes(HugeVal);

        uint Fixed = 0;

        int RetVal;

        if (Radio26Bit.Checked)
          RetVal = GIS_LF_API.TSLF_Write_FSK26Bit(PortHandle, TTYP, Manufacturer, iIdentNr, 0);
        else if (Radio34Bit.Checked)
          RetVal = GIS_LF_API.TSLF_Write_FSK34Bit(PortHandle, TTYP, Manufacturer, iIdentNr, 0);
        else if (Radio36Bit.Checked)
          RetVal = GIS_LF_API.TSLF_Write_FSK36Bit(PortHandle, TTYP, Manufacturer, iIdentNr, Fixed, 0);
        else if (Radio37Bit.Checked)
          RetVal = GIS_LF_API.TSLF_Write_FSK37BitHuge(PortHandle, TTYP, HugeArr,HugeArr.Length, 0);
        else
        {
          MessageBox.Show("No Bit size selected!");
          return;
        }

        if (RetVal < 0) 
        {
          MessageBox.Show("Write Error");
        }
        else
        {
          MessageBox.Show("Write OK");
        }

        // Reset the Antenna Field so Transponder is deselected to make next Read work again
        GIS_LF_API.TSLF_ResetTransponder(PortHandle, 0); 
      }
      else
      {
        MessageBox.Show("No Device attached");
      }
    }
  }
}
