// ReaderAccess.h
// Routines to access GiS Reader with LowLevel Protocol
// Copyright: GiS Gesellschaft fr Informatik und Steuerungstechnik mbH
// Author:    Martin Blank
//
// The communication to the Transponder Read/Write device is don through
// a serial interface with the following parameters:
// Baudrate:  19200
// Databits:      8
// Stopbits:      1
// Parity  :   none

#ifndef __INC_READER_ACCESS_H__
#define __INC_READER_ACCESS_H__

#define NO_ERR        0x00
#define TIMEOUT_ERR   0x01
#define CRC_ERR       0x02
#define DATA_SIZE_ERR 0x03

#define NACK_ERR      0x15
#define SYNC_ERR      0x16
#define CANCEL_ERR    0x18

#ifdef __cplusplus
extern "C" {
#endif

#define TTYP_HITAG_2                     2
#define TTYP_Q5                          3
#define TTYP_HITAG_S                     4
#define TTYP_HITAG_1                     5
#define TTYP_TITAN                       6
#define TTYP_EM4569                      7
#define TTYP_EM4305                      8
#define TTYP_ATA5577                     9
#define TTYP_HITAG_Y                    10
#define TTYP_ATA5575                    11
#define TTYP_SIC7888                    12
#define TTYP_ATA5551                    13


// This function will be called from the reading routine 
// of the serial interface to load the data.
// ch:            Byte read from serial interface
void AddData(unsigned char ch);

// Returns the last received error
int TS_GetLastErr(void);

// Reading the Device Version (pVersion has to have 4 Bytes)
int TS_GetVersion(char * pVersion);

// Get Transponder UID (4 to 8 Byte UID is returned)
int TS_UIDRequest(int TTyp, unsigned char * pUID, int MaxUIDLen);

// Select Tag in Public mode(4 Byte UID is given, only for Hitag Chips)
int TS_Select(int TTyp, unsigned char * pUID,unsigned char * pConfig);

// Set Antenna Field on or off
int TS_SetRF(unsigned char On)

// Read Blocks (multiples of 4 Byte)
// Return value is total read length
int TS_Read(int TTyp, int BlockNr, unsigned char * pData, int Len);

// Write Blocks (multiples of 4 Byte)
int TS_Write(int TTyp, int BlockNr, unsigned char * pData, int DataLen);

#ifdef  __cplusplus
}
#endif

#endif //__INC_READER_FIRMWARE_H__