// ReaderAccessTest.cpp
//

#include <stdio.h>
#include "windows.h"
#include "conio.h"

#define BUF_LEN 20

#include "ReaderAccess.h"

HANDLE handle;

static bool WaitKey(void)
{
  printf("\nPress any key");
  if (getch() == 0x1b) // ESC
    return true;
  return false;
}

int main(int argc, char* argv[])
{
  int i;

  handle = CreateFile("COM6:",GENERIC_READ | GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,0);
  if (handle < 0)
  {
    printf("COM kann nicht geffnet werden\n");
    WaitKey();
    return 1;
  }

  // set the timeout values
  COMMTIMEOUTS CommTimeouts;
  CommTimeouts.ReadIntervalTimeout = 1000;
  CommTimeouts.ReadTotalTimeoutMultiplier = 1000;
  CommTimeouts.ReadTotalTimeoutConstant = 1000;
  CommTimeouts.WriteTotalTimeoutMultiplier = 1000;
  CommTimeouts.WriteTotalTimeoutConstant = 1000;

  // configure
  if (!SetCommTimeouts(handle, &CommTimeouts))
  {
    printf("SetCommTimeouts Error\n");
    WaitKey();
    return 1;
  }

  DCB dcb;
  memset(&dcb,0,sizeof(dcb));
  dcb.DCBlength     = sizeof(dcb);
  dcb.BaudRate      = 19200;
  dcb.ByteSize      = 8;
  dcb.fParity       = FALSE;
  dcb.Parity        = NOPARITY;
  dcb.StopBits      = ONESTOPBIT;

  dcb.fBinary       = TRUE;            // Binrmodus
  dcb.fDtrControl   = DTR_CONTROL_ENABLE;// DTR-Signal nicht verwenden
  dcb.fRtsControl   = RTS_CONTROL_ENABLE;// RTS-Signal nicht verwenden
  dcb.fOutxCtsFlow  = FALSE;              // CTS-Signal nicht verwenden
  dcb.fOutxDsrFlow  = FALSE;              // DSR-Signal nicht verwenden
  if (!SetCommState(handle, &dcb))
  {
    printf("SetCommState Error\n");
    WaitKey();
    return 1;
  }

  char Version[6];
  BYTE UID[8];
  int UIDLen;
  BYTE DataAllgemein[16];

  memset(Version, 0, sizeof(Version));
  if (TS_GetVersion(Version) == 4)
  {
    printf("Version: %4.4s\n", Version);
    if (WaitKey())
      return 1;
  }
  else
  {
    printf("Version not read\n");
    WaitKey();
    return 1;
  }
  // Wait for Tag in field
  while (UIDLen = TS_UIDRequest(TTYP_ATA5577, UID, sizeof(UID))
  {
    printf("Error at UID Request");
    if (WaitKey())
      return 1;
  }

  printf("\nTransponder UID: ");
  for (int i = 0; i < UIDLen; i++)
	printf("%02x ",UID[i]);
  printf("\n");

  memset(DataAllgemein, 0, sizeof(DataAllgemein));
  while (TS_Read(TTYP_ATA5577,1,DataAllgemein,16) != 16)
  {
    printf("Read Error");
    if (WaitKey())
      return 1;
  }
  printf("Data Read at Block %d\n",4);
  for (i = 0; i < 16; i++)
    printf("%02X ", DataAllgemein[i]);
  printf("\n");

  printf("Program finished");
  WaitKey();
  return 0;
}


// To write data via the serial interface, the following function has to be provided
// pData:         pointer to data to send
// Len            Length in bytes of data to send
// return value:  1: successfully sent, 0: not sent
extern "C" int SendPort(unsigned char * pData, int Len)
{
  if (handle > 0)
  {
    DWORD Written = 0;
    if (WriteFile(handle,pData,Len,&Written,NULL))
      return 1;
  }
  return 0;
}

// To enable timeout recognition a routine for time measurement is needed
// returns actual time in Milliseconds
extern "C" unsigned int TimeMillisecond(void)
{
  return GetTickCount();
}

void MessageLoop()
{
  MSG Msg;
  PeekMessage(&Msg, NULL, 0, 0, PM_REMOVE);
}

// Allow Task change to activate other program parts
// This function has to be provided, even if if does nothing
extern "C" void TaskChange(void)
{
  DWORD Errors;
  COMSTAT ComStat;
  BYTE Buff[BUF_LEN];
  DWORD Len;
  DWORD Gelesen;

  MessageLoop();
  // This function checks the serial interface
  if (ClearCommError(handle,&Errors,&ComStat))
  {
    if (ComStat.cbInQue > 0)
    {
      // If data is received
      if (ComStat.cbInQue > BUF_LEN)
        Len = BUF_LEN;
      else
        Len = ComStat.cbInQue;
      if (ReadFile(handle,Buff,Len,&Gelesen,NULL))
      {
        // this is offered to AddData
        for (Len = 0; Len < Gelesen; Len++)
          AddData(Buff[Len]);
      }
    }
  }
}

