﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using rfid;

namespace WasteCS
{
  public partial class Form1 : Form
  {
    int TTYP = GIS_LF_API.TTYP_HITAG_S;

    static string MakeStringFromArray(sbyte[] Arr,int StringNummer)
    {
      string strLine;
      strLine = "";

      int i = 0;
      for (int aktStringNummer = 0; aktStringNummer <= StringNummer; aktStringNummer++)
      {
        strLine = "";
        for (; (i < Arr.Length); i++)
        {
          if (Arr[i] == 0)
          {
            break;
          }
          strLine += (char)Arr[i];
        }
        i++;
        if (strLine.Length == 0)
          break;
      }
      return strLine;
    }

    static sbyte[] MakeArrayFromString(string Name)
    {
      sbyte[] arr = new sbyte[Name.Length];
      for (int i = 0; i < Name.Length; i++)
        arr[i] = (sbyte)Name[i];
      return arr;
    }

    static void CopyArrayTo(SByte[] Src, int SrcOffset, SByte[] Dest, int DestOffset, int Len)
    {
      while (Len > 0)
      {
        Dest[DestOffset++] = Src[SrcOffset++];
        Len--;
      }
    }

    int PortHandle;

    public Form1()
    {
      InitializeComponent();

      int MaxLen = 255;
      int ListLen = 0;
      int DevListLen;

      SByte[] NamenListe = new SByte[1000];
      SByte[] DevList = new SByte[MaxLen];

      DevListLen = GIS_LF_API.TSLF_GetUSBDeviceNames(DevList, MaxLen) - 1;
      CopyArrayTo(DevList, 0, NamenListe, ListLen, DevListLen);
      ListLen += DevListLen;
      DevListLen = GIS_LF_API.TSLF_GetLanDeviceNames(DevList, MaxLen) - 1;
      CopyArrayTo(DevList, 0, NamenListe, ListLen, DevListLen);
      ListLen += DevListLen;
      DevListLen = GIS_LF_API.TSLF_GetCOMDeviceNames(DevList, MaxLen) - 1;
      CopyArrayTo(DevList, 0, NamenListe, ListLen, DevListLen);
      ListLen += DevListLen;
      ListLen++;

      if (ListLen > 1)
      {
        string DeviceName;
        // Use first Device in List
        DeviceName = MakeStringFromArray(NamenListe, 0);

        PortHandle = GIS_LF_API.TSLF_Open(MakeArrayFromString(DeviceName), 19200, 0, 500);
        if (GIS_LF_API.TSLF_IsProgrammer(PortHandle) != 1)
        {
          MessageBox.Show("Device is no Programmer");
          GIS_LF_API.TSLF_Close(PortHandle);
          PortHandle = 0;
        }
        else
        {
          // Get device version from device, to cehck if read device is connected.
          // The first 3 digits of the string give the device number
          // valid device number is 039 for TS-W34AC, 244 for TS-RW38AC
          sbyte[] Version = new sbyte[4];
          sbyte[] InternalDeviceName = new sbyte[32];
          int nReturn = GIS_LF_API.TSLF_GetDeviceVersion(PortHandle, Version, 4,InternalDeviceName,32);
          if(nReturn == -1)
          {
            int Error = GIS_LF_API.TSLF_GetLastError(PortHandle);
            MessageBox.Show("DeviceVersion can not be read");
            GIS_LF_API.TSLF_Close(PortHandle);
            PortHandle = 0;
          }
          else
          {
            if (GIS_LF_API.TSLF_IsPlusProgrammer(PortHandle) != 1)
            {
              // Invalid device, will be closed
              MessageBox.Show("Connected device is no Waste Programmer!");
              GIS_LF_API.TSLF_Close(PortHandle);
              PortHandle = 0;
            }
          }
        }
      }
      else
      {
        PortHandle = 0;
      }

      if (PortHandle <= 0)
      {
        MessageBox.Show("Device could not be opened");
      }
    }

    private void Close_Click(object sender, EventArgs e)
    {
      if (PortHandle > 0)
        GIS_LF_API.TSLF_Close(PortHandle);
      Close();
    }

    private void Read_Click(object sender, EventArgs e)
    {
      if (PortHandle > 0)
      {
        int Manufacturer;
        int iIdentNr;
        if (GIS_LF_API.TSLF_Read_EN14803(PortHandle, out Manufacturer, out iIdentNr) >= 0)
        {
          ManufacturerCode.Text = Manufacturer.ToString();
          IdentNr.Text = iIdentNr.ToString();
        }
        else
        {
          MessageBox.Show("Read Error");
        }
      }
      else
      {
        MessageBox.Show("No Device attached");
      }
    }

    private void Write_Click(object sender, EventArgs e)
    {
      if (PortHandle > 0)
      {
        Byte[] FactoryID = new Byte[8];
        int iIdentNr = 0;
        int iManufacturer = 0;

        if (IdentNr.Text.Length > 0)
          iIdentNr = Convert.ToInt32(IdentNr.Text);
        if (ManufacturerCode.Text.Length > 0)
          iManufacturer = Convert.ToInt32(ManufacturerCode.Text);

        FactoryID[0] = 0;
        FactoryID[1] = 0;
        FactoryID[2] = 0;
        FactoryID[3] = 0;
        FactoryID[4] = 0;
        FactoryID[5] = 0;
        FactoryID[6] = 0;
        FactoryID[7] = 0;

        if (GIS_LF_API.TSLF_Write_EN14803(PortHandle, TTYP, FactoryID, iManufacturer,iIdentNr, 0) < 0) 
        {
          MessageBox.Show("Write Error");
        }
        else
        {
          MessageBox.Show("Write OK");
        }

        // Reset the Antenna Field so Transponder is deselected to make next Read work again
        GIS_LF_API.TSLF_ResetTransponder(PortHandle, 0); 
      }
      else
      {
        MessageBox.Show("No Device attached");
      }
    }
  }
}
