#if !defined(TS_LF_IMPORT_INCLUDED_)
#define TS_LF_IMPORT_INCLUDED_

// Fehler
#define TSLF_NO_ERROR                    0
#define TSLF_PORT_OEFFNEN                1
#define TSLF_TIMEOUT_UNGUELTIG           5
#define TSLF_PORTHANDLE_UNGUELTIG        6
#define TSLF_TIMEOUT                     7
#define TSLF_BUFFER_ZU_KLEIN             8
#define TSLF_BUFFERGROESSE_FALSCH        9
#define TSLF_FALSCHER_TRANSPONDERTYP     10
#define TSLF_KEINE_SENDEDATEN            11
#define TSLF_BLOCKNUMMER_UNGUELTIG       12
#define TSLF_LOCK_UNGUELTIG              13
#define TSLF_CHECKSUMMENFEHLER           14
#define TSLF_KEINE_KOMMUNIKATION         15
#define TSLF_CHECKBUFFER_LEER            16
#define TSLF_NULL_POINTER                17
#define TSLF_MODE_UNGUELTIG              18
#define TSLF_DONT_SET_USB                19 
#define TSLF_COMPARE_ERROR               20 
#define TSLF_NACK                        0x15 // 21
#define TSLF_SYNC                        0x16 // 22
#define TSLF_LOCKED                      0x17 // 23
#define TSLF_CANCEL                      0x18 // 24
#define TSLF_INVALID_DATA                0x19 // 25

#define TTYP_DEFAULT                     0
#define TTYP_HITAG_2                     2
#define TTYP_Q5                          3
#define TTYP_HITAG_S                     4
#define TTYP_HITAG_1                     5
#define TTYP_TITAN                       6
#define TTYP_EM4569                      7
#define TTYP_EM4305                      8
#define TTYP_ATA5577                     9
#define TTYP_HITAG_Y                    10
#define TTYP_ATA5575                    11
#define TTYP_SIC7888                    12
#define TTYP_ATA5551                    13
#define TTYP_EL9265                     14
#define TTYP_EM4582                     15

#define TTYP_HDX_TIRIS                0x21 // 33
#define TTYP_HDX_INCIDE               0x22 // 34
#define TTYP_HDX_PLUS                 0x23 // 35
#define TTYP_HDX_SIC7900              0x24 // 36
#define TTYP_HDX_PLUS_HIQ             0x25 // 37
#define TTYP_HDX_SIC7999              0x26 // 38

#define TTYP_ATA5577_PW               0x30
#define TTYP_EX_BLOCKFOLGE            0x80

#define TTYP_FIRST                       1
#define TTYP_LAST                     0x26

#define SIC_LOGIN_RW                     0x0100

#define READ_AFTER_WRITE                 0x0100
#define READ_AFTER_WRITE_INVERS          0x0200
#define READ_INVERS                      0x0400
#define DO_ALL_ERRORS                    0x0800
#define NO_RETRY                         0x1000
#define DO_OVERLAPPED                    0x2000
#define NO_READ_AFTER_WRITE              0x4000
#define NO_FACTORY_ID                    0x8000

#define ANSWER_MODE_MANCHESTER2KBIT      0x00 // Manchester     RF/64
#define ANSWER_MODE_BIPHASE4KBIT         0x01 // Diff. Biphase  RF/32
#define ANSWER_MODE_FSK2A                0x02 // FSK 2A         RF/50
#define ANSWER_MODE_HDX                  0x03 // HDX
#define ANSWER_MODE_FSK_MANCHESTER       0x04 // FSK 2A         RF/50 + Manchester Dekodierung
#define ANSWER_MODE_MANCHESTER4KBIT      0x05 // Manchester     RF/32
#define ANSWER_MODE_BIPHASE2KBIT         0x06 // Diff. Biphase  RF/64
#define ANSWER_MODE_FSK2A2KBIT           0x07 // FSK 2A         RF/64

#define ANSWER_MODE_INVERS               0x80 // Zustzliches Flag Invers

// Parameter fr TSLF_ReadFormattedData()
#define TSLF_FORMAT_UNIQUE               0x0001
#define TSLF_FORMAT_GIS16BIT             0x0002
#define TSLF_FORMAT_UNIQUEXL             0x0003
#define TSLF_FORMAT_ANIMAL_FDXB          0x0004
#define TSLF_FORMAT_ANIMAL_FDXA          0x0005
#define TSLF_FORMAT_ANIMAL_HDX           0x0006

#define TSLF_READ_BY_DEFAULT             0x0000
#define TSLF_READ_BY_CMD                 0x0100
#define TSLF_READ_UID                    0x0200

// possible values for Lock in WriteFSK commands
#define TSLF_WRITEFSK_LOCK_NONE   0
#define TSLF_WRITEFSK_LOCK_DATA   1
#define TSLF_WRITEFSK_LOCK_CONFIG 2
#define TSLF_WRITEFSK_LOCK_ALL    3

// Rckgabewerte
#define TSLF_RETVAL_ERROR                -1
#define TSLF_RETVAL_OK                   0
#define TSLF_RETVAL_FORMAT_FALSCH        1
#define TSLF_RETVAL_LOCKED               2
#define TSLF_RETVAL_SAME_TAG             3
#define TSLF_RETVAL_PROCESS_PENDING      4
#define TSLF_RETVAL_INVALID_DATA         5

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef _EXPORT
#define _TSLF_DECL __declspec(dllexport)
#else
#ifdef _LF_LIB
#define _TSLF_DECL
#else
#define _TSLF_DECL __declspec(dllimport)
#endif
#endif

// Get Library Version
_TSLF_DECL int __stdcall TSLF_LibVersion(void);
_TSLF_DECL int __stdcall TSLF_SetLog(const char * LogFileName);

_TSLF_DECL int __stdcall TSLF_IsVirtualComInstalled(void);

_TSLF_DECL int __stdcall TSLF_IsDeviceDriverMissing(void);

// Get Names of all USB Devices connected, only valid devices are listed
_TSLF_DECL int __stdcall TSLF_GetUSBDeviceNames(char* NamenListe, int nBufferSize);

// Get Names of all USB Devices connected, all devices from GiS are listed
_TSLF_DECL int __stdcall TSLF_GetUSBDeviceNamesEx(char* NamenListe, int nBufferSize);

// Get Names of all COM Ports available, not tested if there is a device at the port connected.
_TSLF_DECL int __stdcall TSLF_GetCOMDeviceNames(char* NamenListe, int nBufferSize);

// Get Names of all valid LAN devices found in same subnet
_TSLF_DECL int __stdcall TSLF_GetLanDeviceNames(char* NamenListe, int nBufferSize);

// Get Names of all LAN devices found even in other subnets
_TSLF_DECL int __stdcall TSLF_GetLanDeviceNamesEx(char* Buffer, int nBufferSize);

// Change IP Address of LAN device
_TSLF_DECL int __stdcall TSLF_ChangeLanIPAddress(LPCSTR OldAddress, LPCSTR NewAddress, LPCSTR IPMask);

// Change IP Address and Gateway of LAN device
_TSLF_DECL int __stdcall TSLF_ChangeLanIPAddressEx(LPCSTR OldAddress, LPCSTR NewAddress, LPCSTR IPMask, LPCSTR GatewayAddress);

// Get IP Address and Gateway of LAN device
_TSLF_DECL int __stdcall TSLF_GetLanIPAddressEx(LPCSTR strName, char * Address, char * IPMask, char * GatewayAddress);

// Get Information about LAN Device
_TSLF_DECL int __stdcall TSLF_GetLanInfo(LPCSTR strName, char * DHCPName, char * IPAddress, char * IPMask, char * GatewayAddress);

// Get Remote Server IP for LAN device in Client Mode
_TSLF_DECL int __stdcall TSLF_LanGetRemoteIPAndPort(LPCSTR strName, char * pIPAddress, int MaxIPLen, int * pPortNr);

// Set Remote Server IP for LAN device in Client Mode
_TSLF_DECL int __stdcall TSLF_LanWriteRemoteIPAndPort(LPCSTR strName, const char * pIPAddress, int PortNr);

// Check if specific LAN device is available
_TSLF_DECL int __stdcall TSLF_IsLanDeviceAvailable(LPCSTR Address);

// Open LAN Device for configuration
_TSLF_DECL int __stdcall TSLF_LanConfigOpen(LPCSTR strName);

// Set Configuration Record of LAN device, use with care!
_TSLF_DECL int __stdcall TSLF_LanConfigGet(int nHandle, int RecordNr,       BYTE * pData, int Len);

// Get Configuration Record of LAN device
_TSLF_DECL int __stdcall TSLF_LanConfigSet(int nHandle, int RecordNr, const BYTE * pData,int Len);

// Close LAN device configuration.
_TSLF_DECL int __stdcall TSLF_LanConfigClose(int nHandle,int Reset);

// Read Configuration of USB HID device
_TSLF_DECL int __stdcall TSLF_HIDConfigRead(int PortHandle, BYTE * pSendBuf, int SendBufLen,BYTE * pRecvBuf, int RecvBufLen);
// Write Configuration to USB HID device
_TSLF_DECL int __stdcall TSLF_HIDConfigWrite(int PortHandle, BYTE * pSendBuf, int SendBufLen);

_TSLF_DECL int __stdcall TSLF_Open(LPCSTR Name,int Baudrate,int ParityMode, int Timeout);
_TSLF_DECL int __stdcall TSLF_Close(int handle);
_TSLF_DECL int __stdcall TSLF_SetTimeout(int handle, int Timeout);
_TSLF_DECL int __stdcall TSLF_GetPortName(int handle, char * pName, int MaxLen);

_TSLF_DECL int __stdcall TSLF_KeepAppActive(int handle,int bActive);
_TSLF_DECL int __stdcall TSLF_SetReaderAdresse(int handle,int Adresse);

typedef int(__stdcall* ProgressCallback)(DWORD time);
_TSLF_DECL int __stdcall TSLF_SetProgressFct(int handle, ProgressCallback pProgress);

typedef int(__stdcall* AutoReadCallback)(char * pData, int Len);
_TSLF_DECL int __stdcall TSLF_StartAutoRead(int handle, int TermChar, AutoReadCallback pAutoReadProc);
_TSLF_DECL int __stdcall TSLF_StopAutoRead(int handle);

_TSLF_DECL int __stdcall TSLF_GetDeviceVersion(int handle, char * pDevVer, int VerLen,char * pDevName, int NameLen);
_TSLF_DECL int __stdcall TSLF_GetDeviceNumber(int handle);
_TSLF_DECL int __stdcall TSLF_IsProgrammer(int handle);
_TSLF_DECL int __stdcall TSLF_IsHDXProgrammer(int handle);
_TSLF_DECL int __stdcall TSLF_IsAnimalProgrammer(int handle);
_TSLF_DECL int __stdcall TSLF_IsPlusProgrammer(int handle);
_TSLF_DECL int __stdcall TSLF_IsReader(int handle);

_TSLF_DECL int __stdcall TSLF_IsUSB(int handle);
_TSLF_DECL int __stdcall TSLF_GetFilter(int handle,int TTyp, BYTE * pParam,int nParamLen);
_TSLF_DECL int __stdcall TSLF_SetFilter(int handle,int TTyp, BYTE * pParam,int nParamLen);
_TSLF_DECL int __stdcall TSLF_SetBaudrate(int handle,int Baudrate,int ParityMode);
_TSLF_DECL int __stdcall TSLF_SetWriteDelay(int handle,int delay);
_TSLF_DECL int __stdcall TSLF_SetReaderMode(int handle,int mode);
_TSLF_DECL int __stdcall TSLF_GetPowerUpMode(int handle);
_TSLF_DECL int __stdcall TSLF_SetPowerUpMode(int handle,int mode);
_TSLF_DECL int __stdcall TSLF_SetRF(int handle,int OnOff);
_TSLF_DECL int __stdcall TSLF_GetLastError(int handle);
_TSLF_DECL int __stdcall TSLF_SetIO(int handle,int Maske, int Daten);
_TSLF_DECL int __stdcall TSLF_ReadIO(int handle,int * pDaten);
_TSLF_DECL int __stdcall TSLF_TestIO(int handle, int * pTestwert);

_TSLF_DECL int __stdcall TSLF_SetLED(int handle, int red, int green, int yellow);
_TSLF_DECL int __stdcall TSLF_Beep(int handle);

_TSLF_DECL int __stdcall TSLF_ClearLCD(int handle);
_TSLF_DECL int __stdcall TSLF_SetLCDFont(int handle,int FontNr);
_TSLF_DECL int __stdcall TSLF_SetLCDText(int handle, int StartLine, LPCSTR strText);
_TSLF_DECL int __stdcall TSLF_SetLCDLight(int handle, int Helligkeit);

_TSLF_DECL int __stdcall TSLF_ReadSerialNumber(int handle, BYTE * pSerial, int Buflen);
_TSLF_DECL int __stdcall TSLF_SetConfig(int handle,BYTE * pConfig, int ConfigLen);
_TSLF_DECL int __stdcall TSLF_GetConfig(int handle,BYTE * pConfig, int ConfigLen);
_TSLF_DECL int __stdcall TSLF_SetDefault(int handle);
_TSLF_DECL int __stdcall TSLF_SetAnswerMode(int handle,int AnswerMode);
_TSLF_DECL int __stdcall TSLF_GetAnswerMode(int handle);

_TSLF_DECL int __stdcall TSLF_ReadMagnetCard(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_ClearMagnetCard(int handle);

_TSLF_DECL int __stdcall TSLF_GetActivationContext(int handle, char * pData, int Len);
_TSLF_DECL int __stdcall TSLF_WriteActivationContext(int handle, const char * pData, int Len);
_TSLF_DECL int __stdcall TSLF_ReadCounter(int handle, int * pCounter);
_TSLF_DECL int __stdcall TSLF_DecrementCounter(int handle, int * pNewCounter);
_TSLF_DECL int __stdcall TSLF_GetDaysRemain(int handle, int * pDaysRemain);
_TSLF_DECL int __stdcall TSLF_UpdateDaysRemain(int handle, int * pDaysRemain);

_TSLF_DECL int __stdcall TSLF_SetCommunicationMode(int handle, int Mode);
_TSLF_DECL int __stdcall TSLF_RawWrite(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_RawRead(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_Transfer(int PortHandle, int Cmd, BYTE * pSendBuf, int SendBufLen,BYTE * pRecvBuf, int RecvBufLen);
_TSLF_DECL int __stdcall TSLF_TransferEx(int PortHandle, int Cmd, BYTE * pSendBuf, int SendBufLen, BYTE * pRecvBuf, int RecvBufLen, int Timeout);

_TSLF_DECL int __stdcall TSLF_Read (int handle, int TTyp, int WortBlockPageNr,BYTE * pBuffer,int BufLen);
_TSLF_DECL int __stdcall TSLF_Write(int handle, int TTyp, int WortBlockPageNr,BYTE * pBuffer,int BufLen, int Lock);

_TSLF_DECL int __stdcall TSLF_GetTagType(int handle, int TTypExpected);
_TSLF_DECL int __stdcall TSLF_GetTagAnswerMode(int handle, int TTyp, int AnswerModeExpected);

_TSLF_DECL int __stdcall TSLF_Write_Unique(int handle,int TTyp,BYTE * pBuffer,int BufLen, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_FastUnique(int handle, int TTyp, BYTE * pBuffer, int BufLen, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_UniqueXL(int handle,int TTyp,BYTE * pBuffer,int BufLen, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_GiS16Bit(int handle,int TTyp,BYTE * pBuffer,int BufLen, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_GiS12Bit(int handle,int TTyp,BYTE * pBuffer,int BufLen, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_GiS72Bit(int handle,int TTyp,BYTE * pBuffer,int BufLen, int Lock);

_TSLF_DECL int __stdcall TSLF_Write_FSK26Bit(int handle,int TTyp,DWORD FacilityCode, DWORD CardNr, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_FSK33Bit(int handle,int TTyp,DWORD FacilityCode, DWORD CardNr, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_FSK34Bit(int handle,int TTyp,DWORD FacilityCode, DWORD CardNr, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_NC34Bit(int handle, int TTyp, DWORD FacilityCode, DWORD CardNr, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_FSK35Bit(int handle,int TTyp,DWORD FacilityCode, DWORD CardNr, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_FSK36Bit(int handle,int TTyp,DWORD FacilityCode, DWORD CardNr, DWORD FixedField, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_FSK37Bit(int handle,int TTyp,DWORD FacilityCode, DWORD CardNr, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_FSK37BitHuge(int handle,int TTyp, BYTE * pCode, int CodeLen, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_IDCard37Bit(int handle,int TTyp,DWORD FacilityCode, DWORD CardNr, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_IdentiCard37Bit(int handle, int TTyp, DWORD FacilityCode, DWORD CardNr, DWORD FixedField, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_Simplex36Bit(int handle, int TTyp, DWORD FacilityCode, DWORD CardNr, DWORD FixedField, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_AWID26Bit(int handle, int TTyp, DWORD FacilityCode, DWORD CardNr, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_AWID40Bit(int handle, int TTyp, DWORD FacilityCode, DWORD CardNr, int Lock);
_TSLF_DECL int __stdcall TSLF_Write_Kantech39Bit(int handle, int TTyp, DWORD FacilityCode, DWORD CardNr, DWORD Version, int Lock);

_TSLF_DECL int __stdcall TSLF_Read_TTFData(int handle, int AnswerMode, BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_Read_Unique(int handle,BYTE * pBuffer,int BufLen);
_TSLF_DECL int __stdcall TSLF_Read_FastUnique(int handle, BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_Read_UniqueXL(int handle,BYTE * pBuffer,int BufLen);
_TSLF_DECL int __stdcall TSLF_Read_GiS16Bit(int handle,BYTE * pBuffer,int BufLen);
_TSLF_DECL int __stdcall TSLF_Read_GiS12Bit(int handle,BYTE * pBuffer,int BufLen);
_TSLF_DECL int __stdcall TSLF_Read_GiS72Bit(int handle,BYTE * pBuffer,int BufLen);

_TSLF_DECL int __stdcall TSLF_Read_FSK26Bit(int handle,DWORD * pFacilityCode, DWORD * pCardNr);
_TSLF_DECL int __stdcall TSLF_Read_FSK33Bit(int handle,DWORD * pFacilityCode, DWORD * pCardNr);
_TSLF_DECL int __stdcall TSLF_Read_FSK34Bit(int handle,DWORD * pFacilityCode, DWORD * pCardNr);
_TSLF_DECL int __stdcall TSLF_Read_NC34Bit(int handle, DWORD * pFacilityCode, DWORD * pCardNr);
_TSLF_DECL int __stdcall TSLF_Read_FSK35Bit(int handle,DWORD * pFacilityCode, DWORD * pCardNr);
_TSLF_DECL int __stdcall TSLF_Read_FSK36Bit(int handle,DWORD * pFacilityCode, DWORD * pCardNr, DWORD * pFixedField);
_TSLF_DECL int __stdcall TSLF_Read_FSK37Bit(int handle,DWORD * pFacilityCode, DWORD * pCardNr);
_TSLF_DECL int __stdcall TSLF_Read_FSK37BitHuge(int handle,BYTE * pCode, int CodeLen);
_TSLF_DECL int __stdcall TSLF_Read_IDCard37Bit(int handle,DWORD * pFacilityCode, DWORD * pCardNr);
_TSLF_DECL int __stdcall TSLF_Read_IdentiCard37Bit(int handle,DWORD * pFacilityCode, DWORD * pCardNr, DWORD* pFixedField);
_TSLF_DECL int __stdcall TSLF_Read_Simplex36Bit(int handle, DWORD* pFacilityCode, DWORD* pCardNr, DWORD* pFixedField);
_TSLF_DECL int __stdcall TSLF_Read_AWID26Bit(int handle, DWORD * pFacilityCode, DWORD * pCardNr);
_TSLF_DECL int __stdcall TSLF_Read_AWID40Bit(int handle, DWORD * pFacilityCode, DWORD * pCardNr);
_TSLF_DECL int __stdcall TSLF_Read_Kantech39Bit(int handle, DWORD * pFacilityCode, DWORD * pCardNr,DWORD * pVersion);

_TSLF_DECL int __stdcall TSLF_ReadFormattedData(int handle,int TTyp, int ReadType, BYTE * pBuffer, int BufLen);

_TSLF_DECL int __stdcall TSLF_ResetTransponder(int handle, int TTyp);

_TSLF_DECL int __stdcall TSLF_UID_Request(int PortHandle, int TTyp, BYTE * pUID, int BufLen);
_TSLF_DECL int __stdcall TSLF_UID_RequestAC(int PortHandle, int TTyp, BYTE * pUID, int BufLen);
_TSLF_DECL int __stdcall TSLF_UID_RequestNext(int PortHandle, int TTyp, BYTE * pUID, int BufLen);
_TSLF_DECL int __stdcall TSLF_Select(int PortHandle, int TTyp, BYTE * pSelData, int SelDataLen, BYTE * pConfig, int ConfigLen);
_TSLF_DECL int __stdcall TSLF_Authenticate(int handle, int TTyp, BYTE * pKey, int KeyLen, BYTE * pBuffer, int BufLen);

_TSLF_DECL int __stdcall TSLF_WakeUp(int handle, int TTyp, BYTE * pPasswort, int PasswortLen);
_TSLF_DECL int __stdcall TSLF_WakeUpAndRead(int handle, int TTyp, BYTE * pPasswort, int PasswortLen, BYTE * pBuffer, int BufLen);

_TSLF_DECL int __stdcall TSLF_Login(int handle, int TTyp,BYTE * pPW,int nPWLen);
_TSLF_DECL int __stdcall TSLF_WritePW(int handle, int TTyp,BYTE * pOldPW,int OldPWLen,BYTE * pNewPW,int NewPWLen);

_TSLF_DECL int __stdcall TSLF_Disable(int handle, int TTyp);
_TSLF_DECL int __stdcall TSLF_Lock(int handle, int TTyp, int BlockNr);

_TSLF_DECL int __stdcall TSLF_GetTagInfo(int PortHandle, int TTyp, BYTE * pSysInfo, int SysInfoLen);
_TSLF_DECL int __stdcall TSLF_GetTagConfig(int handle, int TTyp, int * pConfig);
_TSLF_DECL int __stdcall TSLF_SetTagConfig(int handle, int TTyp, int Config);

_TSLF_DECL int __stdcall TSLF_SetCTune(int handle, int TTyp, int CTuneVal, int bUseValue, int bOne, int bLock, int * pFrequenz);
_TSLF_DECL int __stdcall TSLF_AutoTune(int handle, int TTyp, int * pFrequenz0, int *pFrequenz1, int bLock);

_TSLF_DECL int __stdcall TSLF_GetTempSensor(int handle, int TTyp, int *pSensorData);
_TSLF_DECL int __stdcall TSLF_StoreTempSensor(int handle, int TTyp, int TimingInfo, int Page, int *pSensorData);
_TSLF_DECL int __stdcall TSLF_GetTempGrad(int handle, int TTyp, int * pTemp);
_TSLF_DECL int __stdcall TSLF_ConvertTemp(int handle, int TTyp, int SensorData);
_TSLF_DECL int __stdcall TSLF_ActivateTempSensor(int handle, int TTyp, int BlockNr);

_TSLF_DECL int __stdcall TSLF_WriteParam(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_ReadParam(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_WritePrefix(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_ReadPrefix(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_WriteSuffix(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_ReadSuffix(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_WriteTermix(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_ReadTermix(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_WritePostcode(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_ReadPostcode(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_WriteReadModeParam(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_ReadReadModeParam(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_WriteHitag2Key(int handle,BYTE * pBuffer, int BufLen);
_TSLF_DECL int __stdcall TSLF_WriteTagKey(int handle, int TTyp, BYTE * pBuffer, int BufLen);

// Functions for ISO11784 (Animal code and Waste management code)
_TSLF_DECL int __stdcall TSLF_Read_FDXA(	int handle, BYTE * pIdNumber, int pIdLen);
_TSLF_DECL int __stdcall TSLF_Write_FDXA(	int handle, int TTyp, BYTE * pFactoryID, BYTE * pIdNumber, int pIdLen, int Lock);

_TSLF_DECL int __stdcall TSLF_Read_FDXB (int handle, int * pCountryCode, BYTE * pBuffer, int BufLen, int *pAnimalFlag, int *pDatenblockFlag, int *pReserved, int *pExtension);
_TSLF_DECL int __stdcall TSLF_Write_FDXB(int handle, int TTyp, BYTE *pFactoryID, int CountryCode, BYTE * pIDNummer, int IDNummerLen, int AnimalFlag, int DatenblockFlag, int Reserved, int Extension, int Lock);

_TSLF_DECL int __stdcall TSLF_Read_HDX (int handle, int * pCountryCode, BYTE * pBuffer, int BufLen, int *pAnimalFlag, int *pDatenblockFlag, int *pReserved, int *pExtension);
_TSLF_DECL int __stdcall TSLF_Write_HDX(int handle, int TTyp, BYTE *pFactoryID, int CountryCode, BYTE * pIDNummer, int IDNummerLen, int AnimalFlag, int DatenblockFlag, int Reserved, int Extension, int Lock);

_TSLF_DECL int __stdcall TSLF_Read_EN14803(int handle, int * pManufacturer, int * pSerialNumber);
_TSLF_DECL int __stdcall TSLF_Read_EN14803HDX(int handle, int * pManufacturer, int * pSerialNumber);
_TSLF_DECL int __stdcall TSLF_Write_EN14803(int handle,int TTyp, BYTE * pFactoryID, int Manufacturer, int SerialNumber, int Lock);

_TSLF_DECL int __stdcall TSLF_IsWriteFinished(int handle, BYTE * pFactoryID);

// Functions specific to EM4582 Transponder
_TSLF_DECL int __stdcall TSLF_EM4582_GetRN(int PortHandle, BYTE * pRN);
_TSLF_DECL int __stdcall TSLF_EM4582_GetIncrement(int PortHandle, DWORD * pData, DWORD * pVal);
_TSLF_DECL int __stdcall TSLF_EM4582_Increment(int PortHandle, BYTE * pPwd, int PWLen, DWORD Data, DWORD Val);
_TSLF_DECL int __stdcall TSLF_EM4582_SendAccess(int PortHandle, BYTE * pID, int IDLen, BYTE * pPwd, int PWLen);
_TSLF_DECL int __stdcall TSLF_EM4582_UnlockKey(int PortHandle, BYTE * pID, int IDLen, BYTE * pPwd, int PWLen);
_TSLF_DECL int __stdcall TSLF_EM4582_SingleAuth(int PortHandle, BYTE * pRN, int RNLen, BYTE *pTokenR, int TokenRLen,BYTE * pTokenT, int MaxTokenTLen);
_TSLF_DECL int __stdcall TSLF_EM4582_MutualAuth(int PortHandle, BYTE * pRN, int RNLen, BYTE *pTokenR, int TokenRLen,BYTE * pTokenT, int MaxTokenTLen);
_TSLF_DECL int __stdcall TSLF_EM4582_IsoMutualAuth(int PortHandle, BYTE *pTokenR, int TokenRLen, BYTE * pTokenT, int MaxTokenTLen);
_TSLF_DECL int __stdcall TSLF_EM4582_SingleAuthDirect(int PortHandle, unsigned int RnTokenTyp, BYTE * pKey, int KeyLen);
_TSLF_DECL int __stdcall TSLF_EM4582_MutualAuthDirect(int PortHandle, unsigned int RnTokenTyp, BYTE * pKey, int KeyLen);
_TSLF_DECL int __stdcall TSLF_EM4582_IsoMutualAuthDirect(int PortHandle, unsigned int RnTokenTyp, BYTE * pKey, int KeyLen);
_TSLF_DECL int __stdcall TSLF_EM4582_SetSKA(int PortHandle, BYTE * pKey, int KeyLen);
_TSLF_DECL int __stdcall TSLF_EM4582_SetSKI(int PortHandle, BYTE * pKey, int KeyLen);
_TSLF_DECL int __stdcall TSLF_EM4582_SetET(int PortHandle, BYTE * pKey, int KeyLen);
_TSLF_DECL int __stdcall TSLF_EM4582_SetER(int PortHandle, BYTE * pKey, int KeyLen);


// Functions only for Data concentrator!
_TSLF_DECL int __stdcall TSLF_GetFaecherState(int PortHandle, int KonzentratorNr,BYTE * pFaecherState,int MaxFaecher);
_TSLF_DECL int __stdcall TSLF_GetFachState(int PortHandle, int KonzentratorNr,int FachNr, BYTE * pFachState,BYTE * pTransponderID,int TransponderIDLength);
_TSLF_DECL int __stdcall TSLF_SetFachState(int PortHandle, int KonzentratorNr,int FachNr, int Steuerinfo, int Dauer);
_TSLF_DECL int __stdcall TSLF_GetFaecherAnzahl(int PortHandle, int KonzentratorNr);

// Functions for Data concentrator and Multiplexer!
_TSLF_DECL int __stdcall TSLF_SetDirectDataMode(int PortHandle, int MasterNr,int FachNr);
_TSLF_DECL int __stdcall TSLF_LeaveDirectDataMode(int PortHandle,int OldAdresse);
_TSLF_DECL int __stdcall TSLF_GetAntennaCount(int handle);

_TSLF_DECL int __stdcall TSLF_SetAntennaPowerPreset(int handle, int Power);
_TSLF_DECL int __stdcall TSLF_GetAntennaPowerPreset(int handle);
_TSLF_DECL int __stdcall TSLF_GetAntennaPower(int handle);


#ifdef __cplusplus
}
#endif

#endif // TS_LF_IMPORT_INCLUDED_
